/*
   Move the definitions of the Comparator
   classes into this file as private classes.
*/
import java.util.*;

public class ComparatorTester1
{
   public static void main(String[] args)
   {
      ArrayList<Country> countries = new ArrayList<Country>();
      countries.add(new Country("Uruguay",  176220, 3.4));
      countries.add(new Country("Thailand", 514000, 67));
      countries.add(new Country("Belgium",   30528, 11));
      countries.add(new Country("Zaire",   2345410, 46.5));
      countries.add(new Country("Belize",    22966, 0.334));


      // Call the sort() method and pass it an anonymous comparator object.

      Collections.sort(countries, new CountryComparatorByName());
      // Now the ArrayList is sorted by country name.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();

      Collections.sort(countries, new CountryComparatorByArea());
      // Now the ArrayList is sorted by area.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();

      Collections.sort(countries, new CountryComparatorByPopulation());
      // Now the ArrayList is sorted by population.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();
   }
}

/**
  These comparators are now private classes (they are not inner classes).
*/
class CountryComparatorByName implements Comparator<Country>
{
   public int compare(Country country1, Country country2)
   {
      return country1.getName().compareTo(country2.getName());
   }
}

class CountryComparatorByArea implements Comparator<Country>
{
   public int compare(Country country1, Country country2)
   {
      if (country1.getArea() < country2.getArea() ) return -1;
      if (country1.getArea() > country2.getArea() ) return  1;
      return 0;
   }
}

class CountryComparatorByPopulation implements Comparator<Country>
{
   public int compare(Country country1, Country country2)
   {
      if (country1.getPopulation() < country2.getPopulation() ) return -1;
      if (country1.getPopulation() > country2.getPopulation() ) return  1;
      return 0;
   }
}


/**
   A country with a name, area, and population.
*/
class Country
{
   private String name;
   private double area;
   private double population;

   /**
      Constructs a country.
      @param name the name of the country
      @param area the area of the country
      @param population  the population of the country
   */
   public Country(String name, double area, double population)
   {
      this.name = name;
      this.area = area;
      this.population = population;
   }

   /**
      Gets the name of the country.
      @return the name
   */
   public String getName()
   {
      return this.name;
   }

   /**
      Gets the area of the country.
      @return the area
   */
   public double getArea()
   {
      return this.area;
   }

   /**
      Gets the population of the country.
      @return the area
   */
   public double getPopulation()
   {
      return this.population;
   }

   public String toString()
   {
      return name + ": area = " + area + ", population = " + population;
   }
}