/*
   Make the definitions of the Comparator classes
   into anonymous inner class at the call to sort().
*/
import java.util.*;

public class ComparatorTester4_fn
{
   public static void main(String[] args)
   {
      ArrayList<Country> countries = new ArrayList<Country>();
      countries.add(new Country("Uruguay",  176220, 3.4));
      countries.add(new Country("Thailand", 514000, 67));
      countries.add(new Country("Belgium",   30528, 11));
      countries.add(new Country("Zaire",   2345410, 46.5));
      countries.add(new Country("Belize",    22966, 0.334));


      // Call sort() and pass it a Comparator that is an
      // anonymous instance of an anonymous inner class.
      Collections.sort(countries, new
         Comparator<Country>()
         {
            public int compare(Country country1, Country country2)
            {
               return country1.getName().compareTo(country2.getName());
            }
         });
      // Now the ArrayList is sorted by name.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();


      // Create "functors" that hold the comparison operations, < or >.
      abstract class Fn  // create a "functor" type
      {
         public abstract boolean fn(double x, double y);
      }
      class Fn1 extends Fn  // create a sub class of the abstract functor
      {
         public boolean fn(double x, double y)
         {
            return x < y;
         }
      }
      class Fn2 extends Fn  // create a sub class of the abstract functor
      {
         public boolean fn(double x, double y)
         {
            return x > y;
         }
      }

      // Instantiate two instances of the functor,
      // one instance using < and one using >.
      // (What happens if fna and fnb are NOT final?)
      final Fn fna = new Fn1();  // using <
      final Fn fnb = new Fn2();  // using <

      Collections.sort(countries, new
         Comparator<Country>()
         {
            public int compare(Country country1, Country country2)
            {   /** fna is a non-local reference! **/
               if ( fna.fn(country1.getArea(), country2.getArea()) ) return -1;
               if (!fna.fn(country1.getArea(), country2.getArea()) ) return  1;
               return 0;
            }
         });
      // Now the ArrayList is sorted by area.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();


      Collections.sort(countries, new
         Comparator<Country>()
         {
            public int compare(Country country1, Country country2)
            {   /** fnb is a non-local reference! **/
               if ( fnb.fn(country1.getArea(), country2.getArea()) ) return -1;
               if (!fnb.fn(country1.getArea(), country2.getArea()) ) return  1;
               return 0;
            }
         });
      // Now the ArrayList is sorted by area.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();


      Collections.sort(countries, new
         Comparator<Country>()
         {
            public int compare(Country country1, Country country2)
            {   /** fna is a non-local reference! **/
               if ( fna.fn(country1.getPopulation(), country2.getPopulation()) ) return -1;
               if (!fna.fn(country1.getPopulation(), country2.getPopulation()) ) return  1;
               return 0;
            }
         });
      // Now the ArrayList is sorted by population.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();

      Collections.sort(countries, new
         Comparator<Country>()
         {
            public int compare(Country country1, Country country2)
            {   /** fnb is a non-local reference! **/
               if ( fnb.fn(country1.getPopulation(), country2.getPopulation()) ) return -1;
               if (!fnb.fn(country1.getPopulation(), country2.getPopulation()) ) return  1;
               return 0;
            }
         });
      // Now the ArrayList is sorted by population.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();
   }
}


/**
   A country with a name, area, and population.
*/
class Country
{
   private String name;
   private double area;
   private double population;

   /**
      Constructs a country.
      @param name the name of the country
      @param area the area of the country
      @param population  the population of the country
   */
   public Country(String name, double area, double population)
   {
      this.name = name;
      this.area = area;
      this.population = population;
   }

   /**
      Gets the name of the country.
      @return the name
   */
   public String getName()
   {
      return this.name;
   }

   /**
      Gets the area of the country.
      @return the area
   */
   public double getArea()
   {
      return this.area;
   }

   /**
      Gets the population of the country.
      @return the area
   */
   public double getPopulation()
   {
      return this.population;
   }

   public String toString()
   {
      return name + ": area = " + area + ", population = " + population;
   }
}